﻿using System.Windows.Forms;

namespace MesicGUI
{
    using System;
    using System.Drawing;
    using PristaniNaMesici;

    public partial class Form1 : Form
    {
        private Game game;
        private int scale = 2;

        public Form1()
        {
            InitializeComponent();
        }

        private void startButton_Click(object sender, EventArgs e)
        {
            this.StartGame();
        }

        private void StartGame()
        {
            var url = urlTextBox.Text;
            urlTextBox.Enabled = false;
            startButton.Enabled = false;
            endButton.Enabled = true;
            leftMotor.Checked = false;
            rightMotor.Checked = false;
            upMotor.Checked = false;
            leftMotor.Enabled = true;
            rightMotor.Enabled = true;
            upMotor.Enabled = true;

            game = new Game(url);
            var response = game.Start();

            this.UpdateEndPosition(response.L, response.R);
            this.UpdatePlayerPosition(response.X, response.Y);
            this.UpdateFlightData(response);
            dataGroupBox.Visible = true;
            player.Visible = true;
            placeForCrash.Visible = true;
            gameTimer.Start();

            if (response.GameResult != null)
            {
                this.EndGame();
                MessageBox.Show(response.GameResult);
            }
        }

        private void EndGame()
        {
            dataGroupBox.Visible = false;
            player.Visible = false;
            gameTimer.Stop();
            this.game = null;
            startButton.Enabled = true;
            urlTextBox.Enabled = true;
            endButton.Enabled = false;
            placeForCrash.Visible = false;
        }

        private void endButton_Click(object sender, EventArgs e)
        {
            this.EndGame();
        }

        private void UpdatePlayerPosition(double x, double y)
        {
            var realX = (int)(Math.Round(x * scale) - 10 * scale);
            var realY = (int)(200 * scale - (Math.Round(y * scale) + 10 * scale));
            player.Location = new Point(realX, realY);
        }

        private void UpdateFlightData(Response response)
        {
            xLabel.Text = $"x: {response.X}";
            yLabel.Text = $"y: {response.Y}";
            pLabel.Text = $"p: {response.P}";
            lLabel.Text = $"l: {response.L}";
            rLabel.Text = $"r: {response.R}";
            vxLabel.Text = $"vx: {response.VX}";
            vyLabel.Text = $"vy: {response.VY}";

            fuelBar.Value = (int)(response.P / 20 * 100);
            if (response.VX > 0)
            {
                leftBar.Value = 0;
                rightBar.Value = (int)Math.Min(response.VX * 2, 100);
            }
            else
            {
                leftBar.Value = (int)Math.Min(-1 * response.VX * 2, 100);
                rightBar.Value = 0;
            }
            if (response.VY > 0)
            {
                upBar.Value = (int) Math.Min(response.VY * 2, 100);
                downBar.Value = 0;
            }
            else
            {
                downBar.Value = (int)Math.Min(-1 * response.VY * 2, 100);
                upBar.Value = 0;
            }
        }

        private void Form1_KeyDown(object sender, KeyEventArgs e)
        {
            
        }

        private void gameTimer_Tick(object sender, EventArgs e)
        {
            if (game != null && game.Id != 0)
            {
                var response = game.Check();

                if (response.GameResult != null)
                {
                    this.EndGame();
                    MessageBox.Show(response.GameResult);
                }
                if (response.P == 0)
                {
                    this.leftMotor.Checked = false;
                    this.rightMotor.Checked = false;
                    this.upMotor.Checked = false;
                    this.leftMotor.Enabled = false;
                    this.rightMotor.Enabled = false;
                    this.upMotor.Enabled = false;
                }

                this.UpdatePlayerPosition(response.X, response.Y);
                this.UpdateFlightData(response);
            }
        }

        private void UpdateEndPosition(double l, double r)
        {
            var startX = scale * l;
            var endX = scale * r;

            var size = new Size((int)(endX - startX), 5);
            var location = new Point((int) startX, 200 * scale - 5);

            placeForCrash.Size = size;
            placeForCrash.Location = location;
        }

        private void leftMotor_CheckedChanged(object sender, EventArgs e)
        {
            if (game != null)
            {
                if (leftMotor.Checked)
                    game.L = 1;
                else
                    game.L = 0;
            }
        }

        private void rightMotor_CheckedChanged(object sender, EventArgs e)
        {
            if (game != null)
            {
                if (rightMotor.Checked)
                    game.R = 1;
                else
                    game.R = 0;
            }
        }

        private void upMotor_CheckedChanged(object sender, EventArgs e)
        {
            if (game != null)
            {
                if (upMotor.Checked)
                    game.U = 1;
                else
                    game.U = 0;
            }
        }

        private void Form1_KeyUp(object sender, KeyEventArgs e)
        {
            switch (e.KeyCode)
            {
                case Keys.Up:
                    upMotor.Checked = !upMotor.Checked;
                    break;
                case Keys.Left:
                    leftMotor.Checked = !leftMotor.Checked;
                    break;
                case Keys.Right:
                    rightMotor.Checked = !rightMotor.Checked;
                    break;
            }
        }
    }
}
